// 定数定義
const CONFIG = {
    API_BASE: '/tools/5league/api',
    SOCKET_URL: 'https://xxo.ooo',
    AUTO_REFRESH_INTERVAL: 30000,
    CHART_UPDATE_INTERVAL: 5000,
    LOCAL_STORAGE_KEYS: {
        IS_ADMIN: 'is_admin',
        AUTO_GRADE: 'auto_grade_enabled',
        AUTO_PUBLISH: 'auto_publish_enabled'
    }
};

// ステート管理
// state の定義部分を探し、以下のように修正:
const state = {
    connected: false,
    currentQuestion: {
        timestamp: null,
        answer: '',
        question: '',
        timeLimit: null
    },
    answers: new Map(),
    questionHistory: [],
    statistics: {
        totalQuestions: 0,
        totalAnswers: 0,
        correctAnswers: 0,
        averageTime: 0
    },
    settings: {
        autoGrade: localStorage.getItem(CONFIG.LOCAL_STORAGE_KEYS.AUTO_GRADE) === 'true',
        autoPublish: localStorage.getItem(CONFIG.LOCAL_STORAGE_KEYS.AUTO_PUBLISH) === 'true'
    }
};

// Socket.IO接続設定
const socket = io(CONFIG.SOCKET_URL, {
    path: '/socket.io/',
    transports: ['websocket', 'polling'],
    upgrade: true,
    rememberUpgrade: true,
    forceNew: true,
    reconnectionAttempts: 5,
    timeout: 10000
});

// UIエレメント
const UI = {
    questionInput: document.getElementById('question-input'),
    answerInput: document.getElementById('answer-input'),
    timeLimit: document.getElementById('time-limit'),
    hintInput: document.getElementById('hint-input'),
    currentQuestion: document.getElementById('current-question-display'),
    answersGrid: document.getElementById('answers-grid'),
    questionHistory: document.getElementById('question-history'),
    connectionStatus: document.getElementById('connection-status'),
    questionTimer: document.getElementById('question-timer'),
    statsChart: document.getElementById('stats-chart'),
    ungradedCount: document.getElementById('ungraded-count'),
    gradedCount: document.getElementById('graded-count'),
    autoGradeMode: document.getElementById('auto-grade-mode'),
    autoPublishResults: document.getElementById('auto-publish-results'),
    lastUpdateTime: document.getElementById('last-update-time')
};

// 統計グラフの初期化
const statsChart = new Chart(UI.statsChart, {
    type: 'line',
    data: {
        labels: [],
        datasets: [
            {
                label: '正答率',
                borderColor: 'rgb(34, 197, 94)',
                data: []
            },
            {
                label: '平均回答時間',
                borderColor: 'rgb(59, 130, 246)',
                data: []
            }
        ]
    },
    options: {
        responsive: true,
        maintainAspectRatio: false,
        scales: {
            y: {
                beginAtZero: true,
                max: 100
            }
        },
        animation: {
            duration: 0
        }
    }
});

// 問題送信
async function sendQuestion() {
    const question = UI.questionInput.value.trim();
    const answer = UI.answerInput.value.trim();
    const timeLimit = parseInt(UI.timeLimit.value) || 60;
    const hint = UI.hintInput.value.trim();

    if (!question || !answer) {
        showNotification('問題と解答を入力してください', 'warning');
        return;
    }

    showLoading();

    try {
        // 前の問題をアーカイブ
        if (state.currentQuestion) {
            state.questionHistory.unshift({
                ...state.currentQuestion,
                answers: Array.from(state.answers.values())
            });
            updateQuestionHistory();
        }

        // 新しい問題を送信
        const questionData = {
            question,
            answer,
            timeLimit,
            hint,
            timestamp: Date.now()
        };

        socket.emit('new_question', questionData);
        
        // フォームをクリア
        UI.questionInput.value = '';
        UI.answerInput.value = '';
        UI.hintInput.value = '';
        UI.timeLimit.value = '60';

        // 状態を更新
        state.currentQuestion = questionData;
        state.answers.clear();
        updateAnswersGrid();
        updateStatistics();
        
        showNotification('問題を送信しました', 'success');
    } catch (error) {
        console.error('Error sending question:', error);
        showNotification('問題の送信に失敗しました', 'error');
    } finally {
        hideLoading();
    }
}

// 回答グリッドの更新
function updateAnswersGrid() {
    if (!UI.answersGrid) return;

    const answersArray = Array.from(state.answers.values());
    const sortedAnswers = answersArray.sort((a, b) => b.timestamp - a.timestamp);

    UI.answersGrid.innerHTML = sortedAnswers.map(answer => `
        <div class="relative p-4 ${answer.correct === null ? 'bg-white' : 
            answer.correct ? 'bg-green-50' : 'bg-red-50'} rounded-lg shadow transition-colors duration-300">
            <div class="absolute top-2 right-2 flex items-center space-x-2">
                <input type="checkbox" 
                    ${answer.correct ? 'checked' : ''} 
                    onchange="markAnswer(${answer.playerId}, this.checked)"
                    class="w-5 h-5 text-green-600 rounded cursor-pointer">
            </div>
            <div class="mb-2 flex justify-between items-center">
                <span class="font-bold">${escapeHTML(answer.playerName)}</span>
                <span class="text-sm text-gray-500">
                    ${new Date(answer.timestamp).toLocaleTimeString()}
                </span>
            </div>
            <img src="${answer.answer}" alt="回答" class="w-full rounded border">
            <div class="mt-2 text-sm flex justify-between items-center">
                <span class="text-gray-500">
                    回答時間: ${formatAnswerTime(answer.answerTime)}
                </span>
                <span class="${answer.correct === null ? 'text-gray-500' : 
                    answer.correct ? 'text-green-600' : 'text-red-600'}">
                    ${answer.correct === null ? '採点待ち' : 
                      answer.correct ? '正解' : '不正解'}
                </span>
            </div>
        </div>
    `).join('');

    // 採点状況を更新
    const ungraded = answersArray.filter(a => a.correct === null).length;
    const graded = answersArray.length - ungraded;
    
    UI.ungradedCount.textContent = ungraded;
    UI.gradedCount.textContent = graded;
}

// 採点処理
async function markAnswer(playerId, isCorrect) {
    if (!playerId) return;

    showLoading();

    try {
        socket.emit('mark_answer', {
            playerId: playerId,
            correct: isCorrect,
            questionId: state.currentQuestion?.id
        });

        // ローカルの状態を更新
        if (state.answers.has(playerId)) {
            const answer = state.answers.get(playerId);
            answer.correct = isCorrect;
            state.answers.set(playerId, answer);
            updateAnswersGrid();
            updateStatistics();
        }

        // 自動公開が有効な場合
        if (state.settings.autoPublish && 
            Array.from(state.answers.values()).every(a => a.correct !== null)) {
            showResults();
        }

    } catch (error) {
        console.error('Error marking answer:', error);
        showNotification('採点処理に失敗しました', 'error');
    } finally {
        hideLoading();
    }
}

// 結果表示
async function showResults() {
    if (state.answers.size === 0) {
        showNotification('表示する結果がありません', 'warning');
        return;
    }

    showLoading();

    try {
        const answers = Array.from(state.answers.values())
            .sort((a, b) => a.teamId - b.teamId || a.timestamp - b.timestamp);

        socket.emit('show_results', { answers });
        showNotification('結果を公開しました', 'success');
    } catch (error) {
        console.error('Error showing results:', error);
        showNotification('結果の公開に失敗しました', 'error');
    } finally {
        hideLoading();
    }
}

// チーム並び替え
async function reorderTeams() {
    showLoading();

    try {
        socket.emit('reorder_teams');
        showNotification('チームの順番を変更しました', 'success');
    } catch (error) {
        console.error('Error reordering teams:', error);
        showNotification('チームの並び替えに失敗しました', 'error');
    } finally {
        hideLoading();
    }
}

// ゲームリセット
async function resetGame() {
    if (!confirm('本当にゲームをリセットしますか？\nすべての得点がクリアされます。')) {
        return;
    }

    showLoading();

    try {
        socket.emit('reset_game');
        state.answers.clear();
        state.questionHistory = [];
        updateAnswersGrid();
        updateQuestionHistory();
        updateStatistics();
        showNotification('ゲームをリセットしました', 'success');
    } catch (error) {
        console.error('Error resetting game:', error);
        showNotification('ゲームのリセットに失敗しました', 'error');
    } finally {
        hideLoading();
    }
}

// 統計情報の更新
function updateStatistics() {
    const answers = Array.from(state.answers.values());
    const graded = answers.filter(a => a.correct !== null);
    const correct = answers.filter(a => a.correct === true);
    
    const stats = {
        totalAnswers: answers.length,
        gradedAnswers: graded.length,
        correctAnswers: correct.length,
        correctRate: graded.length ? (correct.length / graded.length * 100).toFixed(1) : 0,
        averageTime: answers.length ? 
            (answers.reduce((sum, a) => sum + a.answerTime, 0) / answers.length / 1000).toFixed(1) : 0
    };

    // 統計表示を更新
    document.getElementById('correct-answer-rate').textContent = `${stats.correctRate}%`;
    document.getElementById('avg-answer-time').textContent = `${stats.averageTime}秒`;

    // グラフデータを更新
    statsChart.data.labels.push(new Date().toLocaleTimeString());
    statsChart.data.datasets[0].data.push(stats.correctRate);
    statsChart.data.datasets[1].data.push(Math.min(stats.averageTime * 10, 100));

    // 最新の10件のみ表示
    if (statsChart.data.labels.length > 10) {
        statsChart.data.labels.shift();
        statsChart.data.datasets.forEach(dataset => dataset.data.shift());
    }

    statsChart.update();
}

// 問題履歴の更新
function updateQuestionHistory() {
    if (!UI.questionHistory) return;

    UI.questionHistory.innerHTML = state.questionHistory
        .slice(0, 5)  // 最新5件のみ表示
        .map((q, index) => `
            <div class="p-3 ${index === 0 ? 'bg-blue-50' : 'bg-gray-50'} rounded mb-2">
                <div class="flex justify-between items-start mb-1">
                    <div class="font-bold">${escapeHTML(q.question)}</div>
                    <div class="text-sm text-gray-500">
                        ${new Date(q.timestamp).toLocaleTimeString()}
                    </div>
                </div>
                <div class="text-sm text-gray-600">
                    解答: ${escapeHTML(q.answer)}
                </div>
                <div class="text-sm text-gray-500 mt-1">
                    回答数: ${q.answers.length}
                    （正解: ${q.answers.filter(a => a.correct).length}）
                </div>
            </div>
        `).join('');
}

// Socket.IOイベントハンドラ
socket.on('connect', () => {
    console.log('Connected to server');
    state.connected = true;
    updateConnectionStatus(true);
});

socket.on('connect_error', (error) => {
    console.error('Connection Error:', error);
    state.connected = false;
    updateConnectionStatus(false);
});

socket.on('disconnect', (reason) => {
    console.log('Disconnected:', reason);
    state.connected = false;
    updateConnectionStatus(false);
});

// 既存のコードを探す:
socket.on('new_answer', (data) => {
    console.log('New answer received:', data);
    
    // この部分を以下の新しいコードに置き換えてください:

    if (!state.currentQuestion) {
        console.warn('No current question available');
        state.currentQuestion = {
            timestamp: Date.now(),
            answer: ''
        };
    }

    // 回答にタイムスタンプと採点状態を追加
    const answer = {
        ...data,
        timestamp: Date.now(),
        answerTime: state.currentQuestion.timestamp ? 
            Date.now() - state.currentQuestion.timestamp : 
            0,
        correct: state.settings.autoGrade ? 
            (state.currentQuestion.answer && 
             data.answer.toLowerCase() === state.currentQuestion.answer.toLowerCase()) : 
            null
    };

    state.answers.set(data.playerId, answer);
    updateAnswersGrid();
    updateStatistics();

    if (state.settings.autoGrade && state.currentQuestion.answer) {
        markAnswer(data.playerId, answer.correct);
    }
});

socket.on('team_updated', (data) => {
    const teamDisplay = document.getElementById(`team${data.team_id}-players`);
    if (teamDisplay) {
        teamDisplay.innerHTML = data.players
            .sort((a, b) => b.score - a.score)
            .map(player => `
                <div class="player mb-1 px-2 py-1 bg-white rounded flex justify-between items-center">
                    <span>${escapeHTML(player.name)}</span>
                    <span class="text-sm font-bold">${player.score || 0}点</span>
                </div>
            `).join('');

        document.getElementById(`team${data.team_id}-total-score`).textContent = 
            data.players.reduce((sum, p) => sum + (p.score || 0), 0);
    }
});

// 設定の変更ハンドラ
UI.autoGradeMode.addEventListener('change', (e) => {
    state.settings.autoGrade = e.target.checked;
    localStorage.setItem(CONFIG.LOCAL_STORAGE_KEYS.AUTO_GRADE, e.target.checked);
});

UI.autoPublishResults.addEventListener('change', (e) => {
    state.settings.autoPublish = e.target.checked;
    localStorage.setItem(CONFIG.LOCAL_STORAGE_KEYS.AUTO_PUBLISH, e.target.checked);
});

// ユーティリティ関数
function showLoading() {
    document.getElementById('loading-overlay').classList.remove('hidden');
}

function hideLoading() {
    document.getElementById('loading-overlay').classList.add('hidden');
}

// showNotification 関数を探して、以下のように修正:
function showNotification(message, type = 'info') {
    const notification = document.createElement('div');
    notification.className = `
        notification p-4 rounded-lg shadow-lg max-w-md transform transition-all duration-300 ease-in-out
        ${type === 'error' ? 'bg-red-500 text-white' :
          type === 'success' ? 'bg-green-500 text-white' :
          type === 'warning' ? 'bg-yellow-500 text-white' :
          'bg-blue-500 text-white'}
    `;
    notification.style.zIndex = '9999';
    notification.textContent = message;

    const notificationArea = document.getElementById('notification-area');
    if (!notificationArea) {
        console.error('Notification area not found');
        return;
    }

    notificationArea.appendChild(notification);

    // アニメーション
    requestAnimationFrame(() => {
        notification.style.opacity = '1';
        notification.style.transform = 'translateX(0)';
    });

    // 自動削除
    setTimeout(() => {
        notification.style.opacity = '0';
        notification.style.transform = 'translateX(100%)';
        setTimeout(() => {
            if (notification.parentNode) {
                notification.remove();
            }
        }, 300);
    }, 3000);
}

function updateConnectionStatus(connected) {
    UI.connectionStatus.textContent = connected ? '●接続中' : '●切断';
    UI.connectionStatus.className = connected ? 'text-green-500' : 'text-red-500';
}

function formatAnswerTime(ms) {
    const seconds = Math.floor(ms / 1000);
    return `${seconds}秒`;
}

function escapeHTML(str) {
    const div = document.createElement('div');
    div.textContent = str;
    return div.innerHTML;
}

function updateLastUpdateTime() {
    UI.lastUpdateTime.textContent = new Date().toLocaleTimeString();
}

// 問題タイマーの管理
function startQuestionTimer(duration) {
    let timeRemaining = duration;
    UI.questionTimer.textContent = `残り ${timeRemaining} 秒`;

    if (window.questionTimerInterval) {
        clearInterval(window.questionTimerInterval);
    }

    window.questionTimerInterval = setInterval(() => {
        timeRemaining--;
        UI.questionTimer.textContent = `残り ${timeRemaining} 秒`;

        if (timeRemaining <= 0) {
            clearInterval(window.questionTimerInterval);
            UI.questionTimer.textContent = '時間切れ';
            
            // 未回答者を自動的に不正解として処理
            const players = new Set([...document.querySelectorAll('[id^="team"]')]
                .flatMap(team => Array.from(team.querySelectorAll('.player')))
                .map(player => player.textContent.trim()));

            const answeredPlayers = new Set(Array.from(state.answers.values())
                .map(answer => answer.playerName));

            players.forEach(playerName => {
                if (!answeredPlayers.has(playerName)) {
                    // 未回答者として記録
                    state.answers.set(`no_answer_${playerName}`, {
                        playerName,
                        answer: null,
                        timestamp: Date.now(),
                        answerTime: duration * 1000,
                        correct: false
                    });
                }
            });

            updateAnswersGrid();
            updateStatistics();

            if (state.settings.autoPublish) {
                showResults();
            }
        }
    }, 1000);
}

// キーボードショートカットの設定
function setupKeyboardShortcuts() {
    document.addEventListener('keydown', (e) => {
        if (e.ctrlKey || e.metaKey) {
            switch (e.key.toLowerCase()) {
                case 'enter':
                    if (e.shiftKey) {
                        e.preventDefault();
                        sendQuestion();
                    }
                    break;
                case 'p':
                    e.preventDefault();
                    showResults();
                    break;
            }
        }
    });
}

// 定期的な更新処理
function setupAutoRefresh() {
    // 統計情報の自動更新
    setInterval(() => {
        updateStatistics();
        updateLastUpdateTime();
    }, CONFIG.CHART_UPDATE_INTERVAL);

    // オートセーブ
    setInterval(() => {
        localStorage.setItem('admin_state', JSON.stringify({
            currentQuestion: state.currentQuestion,
            questionHistory: state.questionHistory,
            statistics: state.statistics,
            settings: state.settings
        }));
    }, 30000);
}

// 設定の読み込み
function loadSettings() {
    // 自動採点モードの設定を復元
    UI.autoGradeMode.checked = state.settings.autoGrade;
    UI.autoPublishResults.checked = state.settings.autoPublish;

    // 保存された状態の復元
    try {
        const savedState = localStorage.getItem('admin_state');
        if (savedState) {
            const parsed = JSON.parse(savedState);
            Object.assign(state, parsed);
        }
    } catch (error) {
        console.error('Error loading saved state:', error);
    }
}

// アプリケーションの初期化
async function initializeApp() {
    try {
        // 設定の読み込み
        loadSettings();

        // キーボードショートカットの設定
        setupKeyboardShortcuts();

        // 自動更新の設定
        setupAutoRefresh();

        // 問題履歴の更新
        updateQuestionHistory();

        // 統計の初期化
        updateStatistics();

        // 最終更新時刻の設定
        updateLastUpdateTime();

        showNotification('管理画面の準備が完了しました', 'success');
    } catch (error) {
        console.error('Error initializing admin app:', error);
        showNotification('初期化中にエラーが発生しました', 'error');
    }
}

// アプリケーション起動
document.addEventListener('DOMContentLoaded', initializeApp);

// エラーハンドリング
window.onerror = function(msg, url, line, col, error) {
    console.error('Global error:', { msg, url, line, col, error });
    showNotification('予期せぬエラーが発生しました', 'error');
    return false;
};

// Service Worker登録（オフライン対応）
if ('serviceWorker' in navigator) {
    const registerServiceWorker = async () => {
        try {
            const registration = await navigator.serviceWorker.register('/tools/5league/sw.js', {
                scope: '/tools/5league/'
            });
            console.log('Service Worker registered:', registration);
        } catch (error) {
            console.error('Service Worker registration failed:', error);
            // エラーを無視して続行（Service Workerは重要でない機能のため）
        }
    };

    // ページロード後にService Workerを登録
    window.addEventListener('load', registerServiceWorker);
}