// 定数定義
const CONFIG = {
    API_BASE: '/tools/5league/api',
    SOCKET_URL: 'https://xxo.ooo',
    CANVAS_SIZE: {
        MAX_WIDTH: 800,
        ASPECT_RATIO: 1
    },
    BRUSH: {
        DEFAULT_COLOR: '#000000',
        DEFAULT_WIDTH: 3,
        MIN_WIDTH: 1,
        MAX_WIDTH: 20
    },
    RECONNECT_ATTEMPTS: 5,
    HEARTBEAT_INTERVAL: 60000,
    LOCAL_STORAGE_KEYS: {
        PLAYER_ID: 'player_id',
        TEAM_ID: 'team_id',
        PLAYER_NAME: 'player_name',
        DRAWING_DATA: 'temp_drawing'
    }
};

// ステート管理
const state = {
    connected: false,
    currentProblemId: null,
    isDrawing: false,
    canSubmit: true,
    timeRemaining: 0,
    drawingHistory: [],
    historyIndex: -1,
    playerInfo: {
        id: null,
        name: null,
        teamId: null
    }
};

// Socket.IO接続設定
const socket = io(CONFIG.SOCKET_URL, {
    path: '/socket.io/',
    transports: ['websocket', 'polling'],
    upgrade: true,
    rememberUpgrade: true,
    forceNew: true,
    reconnectionAttempts: CONFIG.RECONNECT_ATTEMPTS,
    timeout: 10000,
    auth: {
        playerId: localStorage.getItem(CONFIG.LOCAL_STORAGE_KEYS.PLAYER_ID)
    }
});

// Canvas設定
let canvas = new fabric.Canvas('drawing-board', {
    isDrawingMode: true,
    backgroundColor: 'white',
    width: CONFIG.CANVAS_SIZE.MAX_WIDTH,
    height: CONFIG.CANVAS_SIZE.MAX_WIDTH
});

// UI要素の取得
const UI = {
    clearBtn: document.getElementById('clear-btn'),
    submitBtn: document.getElementById('submit-btn'),
    undoBtn: document.getElementById('undo-btn'),
    redoBtn: document.getElementById('redo-btn'),
    colorPicker: document.getElementById('color-picker'),
    brushSize: document.getElementById('brush-size'),
    resultArea: document.getElementById('result-area'),
    questionText: document.getElementById('question-text'),
    teamScores: {
        team1: document.getElementById('team1-score'),
        team2: document.getElementById('team2-score')
    },
    connectionStatus: document.getElementById('connection-status'),
    timerValue: document.getElementById('timer-value'),
    loadingOverlay: document.getElementById('loading-overlay'),
    offlineWarning: document.getElementById('offline-warning'),
    canvasContainer: document.getElementById('canvas-container')
};

// Canvas初期化
function initializeCanvas() {
    // キャンバスのリサイズ処理
    function resizeCanvas() {
        const containerWidth = UI.canvasContainer.offsetWidth;
        const size = Math.min(containerWidth, CONFIG.CANVAS_SIZE.MAX_WIDTH);
        
        canvas.setWidth(size);
        canvas.setHeight(size);
        canvas.renderAll();

        // スケール調整
        const scale = size / CONFIG.CANVAS_SIZE.MAX_WIDTH;
        canvas.setZoom(scale);
    }

    // 描画設定
    canvas.freeDrawingBrush.width = CONFIG.BRUSH.DEFAULT_WIDTH;
    canvas.freeDrawingBrush.color = CONFIG.BRUSH.DEFAULT_COLOR;

    // イベントリスナー設定
    canvas.on('path:created', function(e) {
        state.drawingHistory = state.drawingHistory.slice(0, state.historyIndex + 1);
        state.drawingHistory.push(JSON.stringify(canvas.toJSON()));
        state.historyIndex++;
        updateUndoRedoButtons();
        saveDrawingState();
    });

    // 初期リサイズとイベントリスナー設定
    resizeCanvas();
    window.addEventListener('resize', debounce(resizeCanvas, 250));

    // 一時保存データの復元
    restoreDrawingState();
}

// 描画状態の保存
function saveDrawingState() {
    try {
        localStorage.setItem(
            CONFIG.LOCAL_STORAGE_KEYS.DRAWING_DATA,
            JSON.stringify({
                canvas: canvas.toJSON(),
                timestamp: Date.now()
            })
        );
    } catch (error) {
        console.error('Failed to save drawing state:', error);
    }
}

// 描画状態の復元
function restoreDrawingState() {
    try {
        const savedData = localStorage.getItem(CONFIG.LOCAL_STORAGE_KEYS.DRAWING_DATA);
        if (savedData) {
            const { canvas: canvasData, timestamp } = JSON.parse(savedData);
            // 30分以内のデータのみ復元
            if (Date.now() - timestamp < 30 * 60 * 1000) {
                canvas.loadFromJSON(canvasData, canvas.renderAll.bind(canvas));
            }
        }
    } catch (error) {
        console.error('Failed to restore drawing state:', error);
    }
}

// Undo/Redo処理
function updateUndoRedoButtons() {
    UI.undoBtn.disabled = state.historyIndex < 0;
    UI.redoBtn.disabled = state.historyIndex >= state.drawingHistory.length - 1;
}

function undo() {
    if (state.historyIndex > 0) {
        state.historyIndex--;
        canvas.loadFromJSON(
            state.drawingHistory[state.historyIndex],
            canvas.renderAll.bind(canvas)
        );
        updateUndoRedoButtons();
    }
}

function redo() {
    if (state.historyIndex < state.drawingHistory.length - 1) {
        state.historyIndex++;
        canvas.loadFromJSON(
            state.drawingHistory[state.historyIndex],
            canvas.renderAll.bind(canvas)
        );
        updateUndoRedoButtons();
    }
}

// チーム情報の表示更新
function updateTeamDisplay(teamId, players) {
    const teamDisplay = document.getElementById(`team${teamId}-players`);
    if (!teamDisplay) return;

    teamDisplay.innerHTML = `
        <div class="text-sm text-gray-600 mb-2">参加者 (${players.length}/5)</div>
        ${players.map(player => `
            <div class="player mb-1 px-2 py-1 bg-gray-100 rounded flex justify-between items-center
                ${player.id === state.playerInfo.id ? 'border-2 border-blue-500' : ''}">
                <span>${escapeHTML(player.name)}</span>
                <span class="text-sm text-gray-600">${player.score || 0}点</span>
            </div>
        `).join('')}
    `;

    // 参加ボタンの状態更新
    const joinButton = document.querySelector(`button[onclick="joinGame(${teamId})"]`);
    if (joinButton) {
        const isFull = players.length >= 5;
        const isCurrentPlayer = players.some(p => p.id === state.playerInfo.id);
        
        joinButton.disabled = isFull || isCurrentPlayer;
        joinButton.textContent = isFull ? '満員' : isCurrentPlayer ? '参加中' : '参加する';
        joinButton.className = `mt-2 px-4 py-2 rounded transition-colors duration-200 ${
            isFull ? 'bg-gray-500' : 
            isCurrentPlayer ? 'bg-green-500' : 
            teamId === 1 ? 'bg-blue-500 hover:bg-blue-600' : 'bg-red-500 hover:bg-red-600'
        } text-white`;
    }
}

// ゲームへの参加処理
async function joinGame(teamId) {
    if (state.playerInfo.id) {
        showNotification('既に参加しています', 'warning');
        return;
    }

    const playerName = await showPrompt('プレイヤー名を入力してください：');
    if (!playerName) return;

    showLoading();

    try {
        const response = await fetch(`${CONFIG.API_BASE}/players`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                name: playerName,
                team_id: teamId
            })
        });

        if (!response.ok) {
            const data = await response.json();
            throw new Error(data.error || '不明なエラーが発生しました');
        }

        const data = await response.json();
        
        // プレイヤー情報を保存
        state.playerInfo = {
            id: data.id,
            name: playerName,
            teamId: teamId
        };

        // LocalStorageに保存
        Object.entries(state.playerInfo).forEach(([key, value]) => {
            localStorage.setItem(CONFIG.LOCAL_STORAGE_KEYS[`PLAYER_${key.toUpperCase()}`], value);
        });

        // Socket.IOに接続情報を送信
        socket.emit('player_joined', state.playerInfo);

        // ハートビート開始
        startHeartbeat();
        await refreshTeams();
        
        showNotification('ゲームに参加しました', 'success');
    } catch (error) {
        console.error('Error joining game:', error);
        showNotification(error.message, 'error');
    } finally {
        hideLoading();
    }
}

// プレイヤーの生存確認（ハートビート）
function startHeartbeat() {
    if (!state.playerInfo.id) return;

    const heartbeatInterval = setInterval(async () => {
        try {
            const response = await fetch(
                `${CONFIG.API_BASE}/players/${state.playerInfo.id}/heartbeat`,
                { method: 'POST' }
            );
            
            if (!response.ok) {
                throw new Error('Heartbeat failed');
            }
        } catch (error) {
            console.error('Heartbeat error:', error);
            clearInterval(heartbeatInterval);
            handleConnectionError();
        }
    }, CONFIG.HEARTBEAT_INTERVAL);

    // ページを閉じる時にクリア
    window.addEventListener('beforeunload', () => {
        clearInterval(heartbeatInterval);
    });
}

// チーム情報の更新
async function refreshTeams() {
    try {
        const teams = await Promise.all([1, 2].map(async teamId => {
            const response = await fetch(`${CONFIG.API_BASE}/teams/${teamId}/players`);
            if (!response.ok) {
                throw new Error('Network response was not ok');
            }
            return response.json();
        }));

        teams.forEach((data, index) => {
            updateTeamDisplay(index + 1, data.players);
        });
    } catch (error) {
        console.error('Error fetching team data:', error);
        showNotification('チーム情報の取得に失敗しました', 'error');
    }
}

async function optimizeImageData(imageData) {
    return new Promise((resolve, reject) => {
        const img = new Image();
        img.onload = () => {
            const canvas = document.createElement('canvas');
            const ctx = canvas.getContext('2d');
            
            // 最大サイズの設定
            const MAX_SIZE = 800;
            let width = img.width;
            let height = img.height;
            
            if (width > MAX_SIZE || height > MAX_SIZE) {
                if (width > height) {
                    height = Math.round(height * (MAX_SIZE / width));
                    width = MAX_SIZE;
                } else {
                    width = Math.round(width * (MAX_SIZE / height));
                    height = MAX_SIZE;
                }
            }
            
            canvas.width = width;
            canvas.height = height;
            ctx.drawImage(img, 0, 0, width, height);
            
            // 品質を0.7に設定して最適化
            resolve(canvas.toDataURL('image/jpeg', 0.7));
        };
        img.onerror = reject;
        img.src = imageData;
    });
}

// 回答送信処理
async function submitAnswer() {
    if (!state.playerInfo.id) {
        showNotification('プレイヤーとして参加してください', 'warning');
        return;
    }

    if (!state.canSubmit) {
        showNotification('既に回答を送信済みです', 'warning');
        return;
    }

    // キャンバスが空かチェック
    const imageData = canvas.toDataURL();
    if (imageData === document.createElement('canvas').toDataURL()) {
        showNotification('回答を入力してください', 'warning');
        return;
    }

    showLoading();
    state.canSubmit = false;
    UI.submitBtn.disabled = true;

    try {
        // 画像データの最適化
        const optimizedImage = await optimizeImageData(imageData);
        
        socket.emit('submit_answer', {
            player_id: state.playerInfo.id,
            answer: optimizedImage,
            timestamp: Date.now()
        });

        showNotification('回答を送信しました', 'success');
        localStorage.removeItem(CONFIG.LOCAL_STORAGE_KEYS.DRAWING_DATA);
    } catch (error) {
        console.error('Error submitting answer:', error);
        showNotification('回答の送信に失敗しました。再度お試しください。', 'error');
        state.canSubmit = true;
        UI.submitBtn.disabled = false;
    } finally {
        hideLoading();
    }
}

// Socket.IOイベントハンドラ
socket.on('connect', () => {
    console.log('Connected to server');
    state.connected = true;
    updateConnectionStatus(true);
    
    if (state.playerInfo.id) {
        socket.emit('player_joined', state.playerInfo);
        startHeartbeat();
    }
});

socket.on('connect_error', (error) => {
    console.error('Connection Error:', error);
    state.connected = false;
    updateConnectionStatus(false);
    handleConnectionError();
});

socket.on('disconnect', (reason) => {
    console.log('Disconnected:', reason);
    state.connected = false;
    updateConnectionStatus(false);
    handleConnectionError();
});

socket.on('team_updated', (data) => {
    updateTeamDisplay(data.team_id, data.players);
});

socket.on('new_question', (data) => {
    state.currentProblemId = data.id;
    state.timeRemaining = data.timeLimit || 60;
    state.canSubmit = true;
    UI.submitBtn.disabled = false;

    UI.questionText.textContent = data.question;
    UI.resultArea.classList.add('hidden');
    
    // キャンバスをリセット
    canvas.clear();
    canvas.backgroundColor = 'white';
    canvas.renderAll();
    
    state.drawingHistory = [];
    state.historyIndex = -1;
    updateUndoRedoButtons();
    
    // タイマー開始
    startTimer();
    
    showNotification('新しい問題が出題されました', 'info');
});

socket.on('answer_result', (data) => {
    if (data.error) {
        showNotification(`エラー: ${data.error}`, 'error');
        state.canSubmit = true;
        UI.submitBtn.disabled = false;
    } else {
        showNotification(
            data.correct ? '正解です！' : `不正解です。正解: ${data.correct_answer}`,
            data.correct ? 'success' : 'error'
        );
    }
});

socket.on('show_results', (data) => {
    const resultsDisplay = document.getElementById('results-display');
    if (!resultsDisplay) return;

    resultsDisplay.innerHTML = data.answers.map(answer => `
        <div class="p-4 ${answer.correct ? 'bg-red-50' : 'bg-blue-50'} rounded-lg">
            <div class="font-bold mb-2">${escapeHTML(answer.playerName)}</div>
            <img src="${answer.answer}" alt="回答" class="w-full rounded border">
            <div class="mt-2 text-sm text-right ${answer.correct ? 'text-red-600' : 'text-blue-600'}">
                ${answer.correct ? '正解' : '不正解'}
            </div>
        </div>
    `).join('');

    UI.resultArea.classList.remove('hidden');
// 結果表示エリアまでスクロール
UI.resultArea.scrollIntoView({ behavior: 'smooth', block: 'nearest' });
});

// タイマー管理
function startTimer() {
    if (window.timeInterval) {
        clearInterval(window.timeInterval);
    }

    UI.timerValue.textContent = state.timeRemaining;
    UI.timerValue.parentElement.classList.remove('hidden');

    window.timeInterval = setInterval(() => {
        state.timeRemaining--;
        UI.timerValue.textContent = state.timeRemaining;

        if (state.timeRemaining <= 0) {
            clearInterval(window.timeInterval);
            if (state.canSubmit) {
                showNotification('時間切れです。回答を自動送信します。', 'warning');
                submitAnswer();
            }
        } else if (state.timeRemaining === 10) {
            showNotification('残り10秒です！', 'warning');
        }
    }, 1000);
}

// 接続状態の更新
function updateConnectionStatus(connected) {
    UI.connectionStatus.textContent = connected ? '接続中' : '切断';
    UI.connectionStatus.className = connected ? 'text-green-500' : 'text-red-500';
    UI.offlineWarning.classList.toggle('hidden', connected);
}

// 接続エラーの処理
function handleConnectionError() {
    showNotification('サーバーとの接続が切断されました。再接続を試みています...', 'error');
}

// イベントリスナーの設定
function setupEventListeners() {
    // キャンバス関連
    UI.clearBtn.addEventListener('click', () => {
        if (confirm('描画内容をクリアしてもよろしいですか？')) {
            canvas.clear();
            canvas.backgroundColor = 'white';
            canvas.renderAll();
            state.drawingHistory = [];
            state.historyIndex = -1;
            updateUndoRedoButtons();
            saveDrawingState();
        }
    });

    UI.submitBtn.addEventListener('click', submitAnswer);
    UI.undoBtn.addEventListener('click', undo);
    UI.redoBtn.addEventListener('click', redo);

    // 描画ツール
    UI.colorPicker.addEventListener('change', (e) => {
        canvas.freeDrawingBrush.color = e.target.value;
    });

    UI.brushSize.addEventListener('input', (e) => {
        canvas.freeDrawingBrush.width = parseInt(e.target.value);
    });

    // キーボードショートカット
    document.addEventListener('keydown', (e) => {
        if (e.ctrlKey || e.metaKey) {
            if (e.key === 'z') {
                e.preventDefault();
                if (e.shiftKey) {
                    redo();
                } else {
                    undo();
                }
            }
        }
    });

    // オフライン検知
    window.addEventListener('online', () => {
        updateConnectionStatus(true);
        socket.connect();
    });

    window.addEventListener('offline', () => {
        updateConnectionStatus(false);
    });

    // ページを閉じる時の処理
    window.addEventListener('beforeunload', async (event) => {
        if (state.playerInfo.id) {
            try {
                await fetch(`${CONFIG.API_BASE}/players/${state.playerInfo.id}/leave`, {
                    method: 'POST',
                    keepalive: true
                });
            } catch (error) {
                console.error('Error leaving game:', error);
            }
        }
    });
}

// ユーティリティ関数
function showLoading() {
    UI.loadingOverlay.classList.remove('hidden');
}

function hideLoading() {
    UI.loadingOverlay.classList.add('hidden');
}

function showNotification(message, type = 'info') {
    const notification = document.createElement('div');
    notification.className = `
        notification p-4 rounded-lg shadow-lg max-w-md transform transition-all duration-300 ease-in-out
        ${type === 'error' ? 'bg-red-500 text-white' :
          type === 'success' ? 'bg-green-500 text-white' :
          type === 'warning' ? 'bg-yellow-500 text-white' :
          'bg-blue-500 text-white'}
    `;
    notification.textContent = message;

    const notificationArea = document.getElementById('notification-area');
    notificationArea.appendChild(notification);

    // アニメーション
    requestAnimationFrame(() => {
        notification.style.opacity = '1';
        notification.style.transform = 'translateX(0)';
    });

    // 自動削除
    setTimeout(() => {
        notification.style.opacity = '0';
        notification.style.transform = 'translateX(100%)';
        setTimeout(() => {
            notification.remove();
        }, 300);
    }, 3000);
}

async function showPrompt(message, defaultValue = '') {
    return new Promise((resolve) => {
        const modalContainer = document.getElementById('modal-container');
        const modalContent = modalContainer.querySelector('.modal-content');

        modalContent.innerHTML = `
            <h3 class="text-lg font-bold mb-4">${message}</h3>
            <input type="text" class="w-full p-2 border rounded mb-4" value="${defaultValue}">
            <div class="flex justify-end space-x-2">
                <button class="px-4 py-2 bg-gray-500 text-white rounded hover:bg-gray-600" data-action="cancel">
                    キャンセル
                </button>
                <button class="px-4 py-2 bg-blue-500 text-white rounded hover:bg-blue-600" data-action="ok">
                    OK
                </button>
            </div>
        `;

        const input = modalContent.querySelector('input');
        input.focus();

        function handleAction(action) {
            modalContainer.classList.add('hidden');
            resolve(action === 'ok' ? input.value : null);
        }

        modalContent.addEventListener('click', (e) => {
            const action = e.target.dataset.action;
            if (action) {
                handleAction(action);
            }
        });

        input.addEventListener('keydown', (e) => {
            if (e.key === 'Enter') {
                handleAction('ok');
            } else if (e.key === 'Escape') {
                handleAction('cancel');
            }
        });

        modalContainer.classList.remove('hidden');
    });
}

function escapeHTML(str) {
    const div = document.createElement('div');
    div.textContent = str;
    return div.innerHTML;
}

function debounce(func, wait) {
    let timeout;
    return function executedFunction(...args) {
        const later = () => {
            clearTimeout(timeout);
            func(...args);
        };
        clearTimeout(timeout);
        timeout = setTimeout(later, wait);
    };
}

// アプリケーションの初期化
async function initializeApp() {
    try {
        // Canvas初期化
        initializeCanvas();

        // イベントリスナー設定
        setupEventListeners();

        // 既存のプレイヤー情報をチェック
        const savedPlayerId = localStorage.getItem(CONFIG.LOCAL_STORAGE_KEYS.PLAYER_ID);
        if (savedPlayerId) {
            const response = await fetch(`${CONFIG.API_BASE}/players/${savedPlayerId}/status`);
            const data = await response.json();

            if (data.active) {
                state.playerInfo = {
                    id: savedPlayerId,
                    name: localStorage.getItem(CONFIG.LOCAL_STORAGE_KEYS.PLAYER_NAME),
                    teamId: parseInt(localStorage.getItem(CONFIG.LOCAL_STORAGE_KEYS.TEAM_ID))
                };
                socket.emit('player_joined', state.playerInfo);
                startHeartbeat();
            } else {
                // 非アクティブな場合は保存データをクリア
                Object.values(CONFIG.LOCAL_STORAGE_KEYS).forEach(key => {
                    localStorage.removeItem(key);
                });
            }
        }

        // チーム情報の初期化
        await refreshTeams();

    } catch (error) {
        console.error('Error initializing app:', error);
        showNotification('アプリケーションの初期化中にエラーが発生しました', 'error');
    }
}

// アプリケーション起動
document.addEventListener('DOMContentLoaded', initializeApp);