const express = require('express');
const http = require('http');
const socketIO = require('socket.io');
const sqlite3 = require('sqlite3').verbose();
const path = require('path');
const cors = require('cors');

const app = express();
const server = http.createServer(app);
const router = express.Router();  // ここに移動

// Socket.IOの設定
const io = socketIO(server, {
    cors: {
        origin: "https://xxo.ooo",
        methods: ["GET", "POST"],
        credentials: true
    },
    path: '/socket.io/'
});

// デバッグログ
const debug = true;
function log(...args) {
    if (debug) console.log(...args);
}

// データベース接続
const db = new sqlite3.Database(path.join(__dirname, '../database/5league.db'));

// テーブルの作成
db.serialize(() => {
    // 問題テーブル
    db.run(`CREATE TABLE IF NOT EXISTS problems (
        id INTEGER PRIMARY KEY AUTOINCREMENT,
        question TEXT NOT NULL,
        answer TEXT NOT NULL,
        category TEXT,
        difficulty INTEGER,
        current BOOLEAN DEFAULT 0
    )`);

    // チームテーブル
    db.run(`CREATE TABLE IF NOT EXISTS teams (
        id INTEGER PRIMARY KEY AUTOINCREMENT,
        name TEXT NOT NULL,
        score INTEGER DEFAULT 0,
        display_order INTEGER DEFAULT 0
    )`);

    // プレイヤーテーブル
    db.run(`CREATE TABLE IF NOT EXISTS players (
        id INTEGER PRIMARY KEY AUTOINCREMENT,
        name TEXT NOT NULL,
        team_id INTEGER,
        score INTEGER DEFAULT 0,
        active BOOLEAN DEFAULT 1,
        last_active DATETIME DEFAULT CURRENT_TIMESTAMP,
        current_answer TEXT,
        current_correct BOOLEAN DEFAULT 0,
        FOREIGN KEY (team_id) REFERENCES teams(id)
    )`);

    // 初期チームの作成（存在しない場合のみ）
    db.get('SELECT COUNT(*) as count FROM teams', [], (err, row) => {
        if (err || row.count === 0) {
            db.run('INSERT INTO teams (name, display_order) VALUES (?, ?)', ['チーム1', 1]);
            db.run('INSERT INTO teams (name, display_order) VALUES (?, ?)', ['チーム2', 2]);
        }
    });
});

// ミドルウェアの設定
app.use(cors({
    origin: 'https://xxo.ooo',
    credentials: true
}));
app.use(express.json({ limit: '50mb' }));
app.use(express.urlencoded({ limit: '50mb', extended: true }));

// プレイヤー作成API
router.post('/players', (req, res) => {
    const { name, team_id } = req.body;
    
    if (!name || !team_id) {
        return res.status(400).json({ error: 'Name and team_id are required' });
    }
    
    if (team_id !== 1 && team_id !== 2) {
        return res.status(400).json({ error: 'Invalid team ID' });
    }

    // アクティブなプレイヤーの重複チェック
    db.get(
        'SELECT id FROM players WHERE name = ? AND active = 1',
        [name],
        (err, existingPlayer) => {
            if (err) {
                console.error('Database error:', err);
                return res.status(500).json({ error: err.message });
            }

            if (existingPlayer) {
                return res.status(400).json({ error: 'このプレイヤー名は既に使用されています' });
            }

            // チームの人数チェック
            db.get(
                'SELECT COUNT(*) as count FROM players WHERE team_id = ? AND active = 1',
                [team_id],
                (err, row) => {
                    if (err) {
                        return res.status(500).json({ error: err.message });
                    }

                    if (row.count >= 5) {
                        return res.status(400).json({ error: 'チームが満員です（最大5人）' });
                    }

                    // プレイヤーを作成
                    db.run(
                        'INSERT INTO players (name, team_id, active, last_active) VALUES (?, ?, 1, CURRENT_TIMESTAMP)',
                        [name, team_id],
                        function(err) {
                            if (err) {
                                return res.status(500).json({ error: err.message });
                            }

                            const playerId = this.lastID;
                            res.json({
                                id: playerId,
                                name: name,
                                team_id: team_id
                            });

                            updateAllTeams();
                        }
                    );
                }
            );
        }
    );
});

// プレイヤーのアクティブ状態を更新
router.post('/players/:id/heartbeat', (req, res) => {
    const playerId = req.params.id;
    db.run(
        'UPDATE players SET last_active = CURRENT_TIMESTAMP WHERE id = ?',
        [playerId],
        (err) => {
            if (err) {
                return res.status(500).json({ error: err.message });
            }
            res.json({ status: 'ok' });
        }
    );
});

// プレイヤーの退出処理
router.post('/players/:id/leave', (req, res) => {
    const playerId = req.params.id;
    
    // プレイヤーを非アクティブにし、チームIDをクリア
    db.run(
        'UPDATE players SET active = 0, team_id = NULL WHERE id = ?',
        [playerId],
        (err) => {
            if (err) {
                return res.status(500).json({ error: err.message });
            }
            res.json({ status: 'ok' });
            updateAllTeams();
        }
    );
});

router.get('/players/:id/status', (req, res) => {
    const playerId = req.params.id;
    
    db.get(
        'SELECT active, team_id FROM players WHERE id = ?',
        [playerId],
        (err, row) => {
            if (err) {
                return res.status(500).json({ error: err.message });
            }
            res.json({
                active: row ? row.active === 1 : false,
                team_id: row ? row.team_id : null
            });
        }
    );
});

// チーム情報の取得
router.get('/teams/:teamId/players', (req, res) => {
    const teamId = req.params.teamId;
    if (teamId !== '1' && teamId !== '2') {
        return res.status(400).json({ error: 'Invalid team ID' });
    }

    db.all(
        'SELECT id, name, score FROM players WHERE team_id = ? AND active = 1 ORDER BY id',
        [teamId],
        (err, players) => {
            if (err) {
                console.error('Database error:', err);
                return res.status(500).json({ error: err.message });
            }
            res.json({ players });
        }
    );
});

// すべてのチーム情報を更新する関数
function updateAllTeams() {
    db.all(
        'SELECT t.id as team_id, t.display_order, t.score as team_score, ' +
        'p.id, p.name, p.score FROM teams t ' +
        'LEFT JOIN players p ON t.id = p.team_id AND p.active = 1 ' +
        'ORDER BY t.display_order, p.id',
        [],
        (err, rows) => {
            if (!err) {
                // チームごとにプレイヤーをグループ化
                const teamData = {};
                rows.forEach(row => {
                    if (!teamData[row.team_id]) {
                        teamData[row.team_id] = {
                            team_id: row.team_id,
                            display_order: row.display_order,
                            team_score: row.team_score,
                            players: []
                        };
                    }
                    if (row.id) {  // プレイヤーが存在する場合
                        teamData[row.team_id].players.push({
                            id: row.id,
                            name: row.name,
                            score: row.score
                        });
                    }
                });

                // 各チームの情報を送信
                Object.values(teamData).forEach(team => {
                    io.emit('team_updated', team);
                });
            }
        }
    );
}

// 現在の問題を取得
function getCurrentQuestion(callback) {
    db.get(
        'SELECT id, question, answer FROM problems WHERE current = 1',
        [],
        (err, row) => {
            callback(err, row);
        }
    );
}

// Socket.IO イベントハンドラ
io.on('connection', (socket) => {
    log('New client connected:', socket.id);

    socket.on('error', (error) => {
        console.error('Socket error:', error);
    });

    socket.on('player_joined', (data) => {
        log('Player joined:', data);
        socket.join(`team_${data.team_id}`);
        
        if (data.player_id) {
            db.run(
                'UPDATE players SET last_active = CURRENT_TIMESTAMP WHERE id = ?',
                [data.player_id]
            );

            // 現在の問題を送信
            getCurrentQuestion((err, question) => {
                if (!err && question) {
                    socket.emit('new_question', { question: question.question });
                }
            });
        }
    });

    socket.on('new_question', (data) => {
        // 現在の問題をリセット
        db.run('UPDATE problems SET current = 0');
        
        // 新しい問題を追加
        db.run(
            'INSERT INTO problems (question, answer, current) VALUES (?, ?, 1)',
            [data.question, data.answer],
            function(err) {
                if (err) {
                    socket.emit('error', { message: 'Failed to save question' });
                    return;
                }

                // プレイヤーの回答をリセット
                db.run('UPDATE players SET current_answer = NULL, current_correct = 0');

                // 全員に新しい問題を送信
                io.emit('new_question', { 
                    question: data.question
                });
            }
        );
    });

// Socket.IO イベントハンドラ内の submit_answer イベント処理を修正
socket.on('submit_answer', (data) => {
    const { player_id, answer } = data;
    if (!player_id) {
        socket.emit('answer_result', { error: 'Player not found' });
        return;
    }

    // プレイヤー情報を取得
    db.get(
        'SELECT name, team_id FROM players WHERE id = ? AND active = 1',
        [player_id],
        (err, player) => {
            if (err || !player) {
                socket.emit('answer_result', { error: 'Player not found' });
                return;
            }

            // 現在の問題を取得
            db.get(
                'SELECT id, answer as correct_answer FROM problems WHERE current = 1',
                [],
                (err, problem) => {
                    if (err) {
                        socket.emit('answer_result', { error: 'Error processing answer' });
                        return;
                    }

                    // 回答を保存
                    db.run(
                        'UPDATE players SET current_answer = ?, current_correct = 0 WHERE id = ?',
                        [answer, player_id],
                        (err) => {
                            if (err) {
                                socket.emit('answer_result', { error: 'Error saving answer' });
                                return;
                            }

                            // 管理画面に新しい回答を通知
                            io.emit('new_answer', {
                                playerId: player_id,
                                playerName: player.name,
                                teamId: player.team_id,
                                answer: answer,
                                correct: false,
                                timestamp: new Date().getTime()
                            });

                            // 回答者に結果を返す
                            socket.emit('answer_result', { 
                                received: true,
                                message: '回答を受け付けました'
                            });

                            // デバッグログ
                            console.log('Answer submitted:', {
                                playerId: player_id,
                                playerName: player.name,
                                teamId: player.team_id
                            });
                        }
                    );
                }
            );
        }
    );
});

// 採点処理の改良
socket.on('mark_answer', (data) => {
    const { playerId, correct } = data;
    console.log('Marking answer:', { playerId, correct }); // デバッグログ
    
    db.run(
        'UPDATE players SET current_correct = ? WHERE id = ?',
        [correct ? 1 : 0, playerId],
        (err) => {
            if (err) {
                console.error('Error marking answer:', err);
                return;
            }

            // スコアの更新（正解の場合）
            if (correct) {
                db.run('UPDATE players SET score = score + 1 WHERE id = ?', [playerId]);
                db.run(
                    'UPDATE teams SET score = score + 1 WHERE id = (SELECT team_id FROM players WHERE id = ?)',
                    [playerId]
                );
            }

            // 採点結果を全員に通知
            io.emit('answer_marked', {
                playerId: playerId,
                correct: correct
            });

            updateAllTeams();
            console.log('Answer marked:', { playerId, correct }); // デバッグログ
        }
    );
});

    function emitUpdatedResults() {
        db.all(
            `SELECT 
                p.name as playerName, 
                p.current_answer as answer, 
                p.current_correct as correct,
                t.id as team_id
             FROM players p
             LEFT JOIN teams t ON p.team_id = t.id
             WHERE p.active = 1 
             AND p.current_answer IS NOT NULL 
             ORDER BY t.display_order, p.id`,
            [],
            (err, answers) => {
                if (!err) {
                    const orderedAnswers = answers.map(answer => ({
                        playerName: answer.playerName,
                        answer: answer.answer,
                        correct: answer.correct === 1,
                        teamId: answer.team_id
                    }));
    
                    io.emit('show_results', { answers: orderedAnswers });
                }
            }
        );
    }

    socket.on('show_results', () => {
        db.all(
            `SELECT 
                p.name as playerName, 
                p.current_answer as answer, 
                p.current_correct as correct,
                t.id as team_id
             FROM players p
             LEFT JOIN teams t ON p.team_id = t.id
             WHERE p.active = 1 
             AND p.current_answer IS NOT NULL 
             ORDER BY t.display_order, p.id`,
            [],
            (err, answers) => {
                if (!err) {
                    // 結果をチーム順、プレイヤー順に整理
                    const orderedAnswers = answers.map(answer => ({
                        playerName: answer.playerName,
                        answer: answer.answer,
                        correct: answer.correct === 1,
                        teamId: answer.team_id
                    }));
    
                    io.emit('show_results', { answers: orderedAnswers });
                }
            }
        );
    });

    socket.on('reorder_teams', () => {
        db.run('UPDATE teams SET display_order = CASE id WHEN 1 THEN 2 WHEN 2 THEN 1 END');
        updateAllTeams();
    });

    socket.on('disconnect', () => {
        log('Client disconnected:', socket.id);
    });
});

// 静的ファイルの提供
app.use('/tools/5league', express.static(path.join(__dirname, '../public')));

// APIルーターのマウント
app.use('/tools/5league/api', router);

const PORT = process.env.PORT || 3000;
server.listen(PORT, () => {
    console.log(`Server running on port ${PORT}`);
});